"use strict";

var _fs = _interopRequireDefault(require("fs"));
var _getCredentials = require("@commercetools/get-credentials");
var _npmlog = _interopRequireDefault(require("npmlog"));
var _prettyError = _interopRequireDefault(require("pretty-error"));
var _yargs = _interopRequireDefault(require("yargs"));
var _constants = _interopRequireDefault(require("./constants"));
var _main = _interopRequireDefault(require("./main"));
var _this = void 0;
function _interopRequireDefault(e) { return e && e.__esModule ? e : { "default": e }; }
process.title = 'csvparserprice';
var args = _yargs["default"].usage("\n\nUsage: $0 [options]\nConverts commercetools states from CSV to JSON").showHelpOnFail(false).option('inputFile', {
  alias: 'i',
  "default": 'stdin',
  describe: 'Path to input CSV file.'
}).coerce('inputFile', function (arg) {
  if (arg !== 'stdin') return _fs["default"].createReadStream(String(arg));
  return process.stdin;
}).option('outputFile', {
  alias: 'o',
  "default": 'stdout',
  describe: 'Path to output JSON file.'
}).coerce('outputFile', function (arg) {
  if (arg !== 'stdout') return _fs["default"].createWriteStream(String(arg));
  return process.stdout;
}).option('apiUrl', {
  "default": _constants["default"].host.api,
  describe: 'The host URL of the HTTP API service.'
}).option('authUrl', {
  "default": _constants["default"].host.auth,
  describe: 'The host URL of the OAuth API service.'
}).option('batchSize', {
  alias: 'b',
  "default": _constants["default"].standardOption.batchSize,
  describe: 'Number of CSV rows to handle simultaneously.'
}).option('delimiter', {
  alias: 'd',
  "default": _constants["default"].standardOption.delimiter,
  describe: 'Used CSV delimiter.'
}).option('accessToken', {
  describe: 'CTP client access token'
}).option('projectKey', {
  alias: 'p',
  describe: 'API project key.',
  demand: true
}).option('logLevel', {
  "default": 'info',
  describe: 'Logging level: error, warn, info or verbose.'
}).option('logFile', {
  "default": _constants["default"].standardOption.defaultLogFile,
  describe: 'Path to file where to save logs.'
}).coerce('logLevel', function (arg) {
  _npmlog["default"].level = arg;
}).argv;
var logError = function logError(error) {
  var errorFormatter = new _prettyError["default"]();
  if (_npmlog["default"].level === 'verbose') process.stderr.write("ERR: ".concat(errorFormatter.render(error), " \n"));
  // Add new line to make it intuitive
  else process.stderr.write("ERR: ".concat(error.message || error, " \n")); // Add new line to make it intuitive
};
var errorHandler = function errorHandler(errors) {
  // print errors to stderr if we use stdout for data output
  // if we save data to output file errors are already logged by npmlog
  if (Array.isArray(errors)) errors.forEach(logError);else logError(errors);
  process.exitCode = 1;
};
var resolveCredentials = function resolveCredentials(_args) {
  if (_args.accessToken) return Promise.resolve({});
  return (0, _getCredentials.getCredentials)(_args.projectKey);
};

// If the stdout is used for a data output, save all logs to a log file.
if (args.outputFile === process.stdout) _npmlog["default"].stream = _fs["default"].createWriteStream(args.logFile);

// Register error listener
args.outputFile.on('error', errorHandler);
resolveCredentials(args).then(function (credentials) {
  return new _main["default"]({
    apiConfig: {
      host: args.authUrl,
      apiUrl: args.apiUrl,
      projectKey: args.projectKey,
      credentials: credentials
    },
    accessToken: args.accessToken,
    logger: {
      error: _npmlog["default"].error.bind(_this, ''),
      warn: _npmlog["default"].warn.bind(_this, ''),
      info: _npmlog["default"].info.bind(_this, ''),
      verbose: _npmlog["default"].verbose.bind(_this, '')
    },
    csvConfig: {
      delimiter: args.delimiter,
      batchSize: args.batchSize
    }
  });
}).then(function (csvParserPrice) {
  return csvParserPrice.parse(args.inputFile, args.outputFile);
});