"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var _csvParser = _interopRequireDefault(require("csv-parser"));
var _highland = _interopRequireDefault(require("highland"));
var _JSONStream = _interopRequireDefault(require("JSONStream"));
var _lodash = _interopRequireDefault(require("lodash.pick"));
var _lodash2 = _interopRequireDefault(require("lodash.mapvalues"));
var _lodash3 = _interopRequireDefault(require("lodash.memoize"));
var _lodash4 = _interopRequireDefault(require("lodash.isnil"));
var _flat = require("flat");
var _nodeFetch = _interopRequireDefault(require("node-fetch"));
var _stripBomStream = _interopRequireDefault(require("strip-bom-stream"));
var _sdkMiddlewareAuth = require("@commercetools/sdk-middleware-auth");
var _sdkClient = require("@commercetools/sdk-client");
var _sdkMiddlewareHttp = require("@commercetools/sdk-middleware-http");
var _apiRequestBuilder = require("@commercetools/api-request-builder");
var _sdkMiddlewareUserAgent = require("@commercetools/sdk-middleware-user-agent");
var _constants = _interopRequireDefault(require("./constants"));
var _mapCustomFields = _interopRequireDefault(require("./map-custom-fields"));
var _package = require("../package.json");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { "default": e }; }
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _objectWithoutProperties(e, t) { if (null == e) return {}; var o, r, i = _objectWithoutPropertiesLoose(e, t); if (Object.getOwnPropertySymbols) { var n = Object.getOwnPropertySymbols(e); for (r = 0; r < n.length; r++) o = n[r], -1 === t.indexOf(o) && {}.propertyIsEnumerable.call(e, o) && (i[o] = e[o]); } return i; }
function _objectWithoutPropertiesLoose(r, e) { if (null == r) return {}; var t = {}; for (var n in r) if ({}.hasOwnProperty.call(r, n)) { if (-1 !== e.indexOf(n)) continue; t[n] = r[n]; } return t; }
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _classCallCheck(a, n) { if (!(a instanceof n)) throw new TypeError("Cannot call a class as a function"); }
function _defineProperties(e, r) { for (var t = 0; t < r.length; t++) { var o = r[t]; o.enumerable = o.enumerable || !1, o.configurable = !0, "value" in o && (o.writable = !0), Object.defineProperty(e, _toPropertyKey(o.key), o); } }
function _createClass(e, r, t) { return r && _defineProperties(e.prototype, r), t && _defineProperties(e, t), Object.defineProperty(e, "prototype", { writable: !1 }), e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
var CsvParserPrice = exports["default"] = /*#__PURE__*/function () {
  function CsvParserPrice(_ref) {
    var apiConfig = _ref.apiConfig,
      logger = _ref.logger,
      _ref$csvConfig = _ref.csvConfig,
      csvConfig = _ref$csvConfig === void 0 ? {} : _ref$csvConfig,
      accessToken = _ref.accessToken;
    _classCallCheck(this, CsvParserPrice);
    this.client = (0, _sdkClient.createClient)({
      middlewares: [(0, _sdkMiddlewareAuth.createAuthMiddlewareForClientCredentialsFlow)(_objectSpread(_objectSpread({}, apiConfig), {}, {
        fetch: _nodeFetch["default"]
      })), (0, _sdkMiddlewareUserAgent.createUserAgentMiddleware)({
        libraryName: 'csv-parser-price',
        libraryVersion: _package.version
      }), (0, _sdkMiddlewareHttp.createHttpMiddleware)({
        host: apiConfig.apiUrl,
        fetch: _nodeFetch["default"]
      })]
    });
    this.apiConfig = apiConfig;
    this.accessToken = accessToken;

    // noop the logger if not defined, should not polute stdout
    this.logger = logger || {
      error: function error() {},
      warn: function warn() {},
      info: function info() {},
      verbose: function verbose() {}
    };
    this.batchSize = csvConfig.batchSize || _constants["default"].standardOption.batchSize;
    this.delimiter = csvConfig.delimiter || _constants["default"].standardOption.delimiter;
  }
  return _createClass(CsvParserPrice, [{
    key: "parse",
    value: function parse(input, output) {
      var _this = this;
      this.logger.info('Starting conversion');
      var rowIndex = 2;
      (0, _highland["default"])(input)
      // Parse CSV return each row as object
      .through((0, _stripBomStream["default"])()).through((0, _csvParser["default"])({
        separator: this.delimiter,
        strict: true
      })).map(function (a) {
        if (!a[_constants["default"].header.sku]) {
          throw new Error("".concat(_constants["default"].header.sku, " key is missing/mis-spelled in the csv header."));
        }
        return a;
      }).errors(function (err, push) {
        push(err.message);
      }).stopOnError(function (err) {
        _this.logger.error(err);
        output.emit('error', err);
      })
      // Sort by SKU so later when reducing prices we can easily group by SKU
      .sortBy(function (a, b) {
        return a[_constants["default"].header.sku].localeCompare(b[_constants["default"].header.sku]);
      })
      // Limit amount of rows to be handled at the same time
      .batch(this.batchSize).stopOnError(function (err) {
        _this.logger.error(err);
        output.emit('error', err);
      }).flatMap(_highland["default"])
      // Unflatten object keys with a dot to nested values
      .map(_flat.unflatten).map(this.transformPriceData).map(this.renameHeaders).flatMap(function (data) {
        return (0, _highland["default"])(_this.transformCustomData(data, rowIndex));
      }).map(this.deleteMovedData).doto(function () {
        _this.logger.verbose("Processed row ".concat(rowIndex));
        rowIndex += 1;
      }).stopOnError(function (err) {
        _this.logger.error(err);
        output.emit('error', err);
      }).reduce({
        prices: []
      }, this.mergeBySku).doto(function (data) {
        var numberOfPrices = Number(JSON.stringify(data.length)) + 1;
        _this.logger.info("Done with conversion of ".concat(numberOfPrices, " prices"));
      }).pipe(_JSONStream["default"].stringify(false)).pipe(output);
    }

    // Transform price values to the type the API expects
    // eslint-disable-next-line class-methods-use-this
  }, {
    key: "transformPriceData",
    value: function transformPriceData(price) {
      return (0, _lodash2["default"])(price, function (value) {
        if (value.centAmount) {
          var optFields = (0, _lodash["default"])(value, ['type', 'fractionDigits', 'preciseAmount']);
          if (optFields.fractionDigits) optFields.fractionDigits = Number(optFields.fractionDigits);

          // don't propagate preciseAmount property if it is empty
          if ((0, _lodash4["default"])(optFields.preciseAmount) || optFields.preciseAmount === '') delete optFields.preciseAmount;else optFields.preciseAmount = Number(optFields.preciseAmount);
          return _objectSpread({
            centAmount: Number(value.centAmount),
            currencyCode: value.currencyCode
          }, optFields);
        }
        return value;
      });
    }
  }, {
    key: "transformCustomData",
    value: function transformCustomData(price, rowIndex) {
      if (price.customType) {
        this.logger.verbose("Found custom type ".concat(price.customType, ", row ").concat(rowIndex));
        return this.processCustomField(price, rowIndex)
        // Using arrow without body trips up babel transform-object-rest-spread
        // eslint-disable-next-line arrow-body-style
        .then(function (customTypeObj) {
          return _objectSpread(_objectSpread({}, price), {}, {
            custom: customTypeObj
          });
        });
      }
      return Promise.resolve(price);
    }

    // Rename for compatibility with price import module
    // eslint-disable-next-line class-methods-use-this
  }, {
    key: "renameHeaders",
    value: function renameHeaders(price) {
      var _newPrice$customerGro, _newPrice$channel;
      var newPrice = _objectSpread({}, price);
      if ((_newPrice$customerGro = newPrice.customerGroup) !== null && _newPrice$customerGro !== void 0 && _newPrice$customerGro.groupName) {
        newPrice.customerGroup.id = newPrice.customerGroup.groupName;
        delete newPrice.customerGroup.groupName;
      }
      if ((_newPrice$channel = newPrice.channel) !== null && _newPrice$channel !== void 0 && _newPrice$channel.key) {
        newPrice.channel.id = newPrice.channel.key;
        delete newPrice.channel.key;
      }
      return newPrice;
    }

    // eslint-disable-next-line class-methods-use-this
  }, {
    key: "mergeBySku",
    value: function mergeBySku(data, currentPrice) {
      var previousPrice = data.prices[data.prices.length - 1];

      // remove variant-sku property from price object
      var _CONSTANTS$header$sku = _constants["default"].header.sku,
        currentSku = currentPrice[_CONSTANTS$header$sku],
        restOfCurrentPrice = _objectWithoutProperties(currentPrice, [_CONSTANTS$header$sku].map(_toPropertyKey));
      if ((previousPrice === null || previousPrice === void 0 ? void 0 : previousPrice.sku) === currentSku) previousPrice.prices.push(restOfCurrentPrice);else data.prices.push({
        sku: currentSku,
        prices: [restOfCurrentPrice]
      });
      return data;
    }

    // eslint-disable-next-line class-methods-use-this
  }, {
    key: "deleteMovedData",
    value: function deleteMovedData(price) {
      var newPrice = _objectSpread({}, price);
      if (newPrice.customType) delete newPrice.customType;
      if (newPrice.customField) delete newPrice.customField;
      return newPrice;
    }

    // Convert custom type value to the expected native type
  }, {
    key: "processCustomField",
    value: function processCustomField(data, rowIndex) {
      var _this2 = this;
      return this.getCustomTypeDefinition(data.customType).then(function (customType) {
        _this2.logger.info("Fetched custom type ".concat(customType.key, " definition"));
        var customTypeObj = _mapCustomFields["default"].parse(data.customField, customType, rowIndex);
        if (customTypeObj.error.length) return Promise.reject(customTypeObj.error);
        return customTypeObj.data;
      });
    }
  }]);
}(); // Easiest way to wrap the getCustomTypeDefinition in the memoize method
CsvParserPrice.prototype.getCustomTypeDefinition = (0, _lodash3["default"])(function _getCustomTypeDefinition(customTypeKey) {
  var getTypeByKeyUri = (0, _apiRequestBuilder.createRequestBuilder)({
    projectKey: this.apiConfig.projectKey
  }).types.where("key=\"".concat(customTypeKey, "\"")).build();
  var execObj = {
    uri: getTypeByKeyUri,
    method: 'GET'
  };
  if (this.accessToken) execObj.headers = {
    Authorization: "Bearer ".concat(this.accessToken)
  };
  return this.client.execute(execObj).then(function (response) {
    if (response.body.count === 0) return Promise.reject(new Error("No type with key '".concat(customTypeKey, "' found")));
    return response.body.results[0];
  });
});