"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;
var _lodash = _interopRequireDefault(require("lodash.pick"));
var _lodash2 = _interopRequireDefault(require("lodash.mapvalues"));
var _constants = _interopRequireDefault(require("./constants"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { "default": e }; }
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _toConsumableArray(r) { return _arrayWithoutHoles(r) || _iterableToArray(r) || _unsupportedIterableToArray(r) || _nonIterableSpread(); }
function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _iterableToArray(r) { if ("undefined" != typeof Symbol && null != r[Symbol.iterator] || null != r["@@iterator"]) return Array.from(r); }
function _arrayWithoutHoles(r) { if (Array.isArray(r)) return _arrayLikeToArray(r); }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
var _default = exports["default"] = function MapCustomFields() {
  function isValidValue(value) {
    return typeof value === 'string' && value.length > 0;
  }
  function processError(errors, rowIndex, customTypeKey) {
    if (!Array.isArray(errors)) return new Error("[row ".concat(rowIndex, ": ").concat(customTypeKey, "] - ").concat(errors));
    return errors.reduce(function (prev, curr) {
      prev.push(new Error("[row ".concat(rowIndex, ": ").concat(customTypeKey, "] - ").concat(curr)));
      return prev;
    }, []);
  }
  function parse(data, customType, rowIndex) {
    var _this = this;
    var custom = {
      type: {
        id: customType.id
      },
      fields: {}
    };
    var result = {
      error: [],
      data: custom
    };
    Object.keys(data).forEach(function (key) {
      var value = data[key];
      customType.fieldDefinitions.forEach(function (fieldDef) {
        if (fieldDef.name === key) switch (fieldDef.type.name) {
          case 'Number':
            {
              var _result = _this.mapNumber(value);
              if (_result.error) result.error.push(processError(_result.error, rowIndex, customType.key));
              if (!(_result.data === undefined)) custom.fields[key] = _result.data;
              break;
            }
          case 'Boolean':
            {
              var _result2 = _this.mapBoolean(value);
              if (_result2.error) result.error.push(processError(_result2.error, rowIndex, customType.key));
              if (!(_result2.data === undefined)) custom.fields[key] = _result2.data;
              break;
            }
          case 'Money':
            {
              var _result3 = _this.mapMoney(value);
              if (_result3.error) result.error.push(processError(_result3.error, rowIndex, customType.key));
              if (!(_result3.data === undefined)) custom.fields[key] = _result3.data;
              break;
            }
          case 'Set':
            {
              var _result$error;
              var _result4 = _this.mapSet(value, fieldDef.type.elementType);
              if (_result4.error.length) (_result$error = result.error).push.apply(_result$error, _toConsumableArray(processError(_result4.error, rowIndex, customType.key)));
              if (_result4.data.length) custom.fields[key] = _result4.data;
              break;
            }
          case 'String':
          case 'Enum':
          case 'LocalizedEnum':
          case 'LocalizedString':
          case 'Date':
          case 'Time':
          case 'DateTime':
          case 'Reference':
            {
              if (!(value === undefined)) custom.fields[key] = value;
              break;
            }
          default:
            {
              var unsupportedMsg = "'".concat(fieldDef.type.name, "' type is not supported! Kindly raise an issue for this");
              result.error.push(new Error("[row ".concat(rowIndex, ": ").concat(customType.key, "] - ").concat(unsupportedMsg)));
            }
        }
      });
    });
    return result;
  }
  function mapBoolean(value) {
    var result = {};
    if (!isValidValue(value)) return result;
    var _value = value.trim();
    var errorMsg = "The value '".concat(_value, "' is not a valid boolean value");
    try {
      var b = JSON.parse(_value.toLowerCase());
      if (!(typeof b === 'boolean')) {
        result.error = errorMsg;
        return result;
      }
      result.data = b;
      return result;
    } catch (error) {
      result.error = errorMsg;
      return result;
    }
  }

  /**
   * Takes a value and returns a CTP money object
   * @param value String "USD 123" or object {centAmount: '123', currencyCode: 'USD'}
   * @returns {{ data: CTP price object, error: Error message if any }}
   */
  function mapMoney(value) {
    var result = {};

    // handle money value provided as an object
    if (_typeof(value) === 'object' && value.centAmount && value.currencyCode) {
      var numberKeys = ['centAmount', 'fractionDigits', 'preciseAmount'];
      var data = (0, _lodash2["default"])(
      // pick only Money properties
      (0, _lodash["default"])(value, ['currencyCode', 'type'].concat(numberKeys)),
      // map Number properties to integer
      function (val, key) {
        return numberKeys.includes(key) ? parseInt(val, 10) : val;
      });
      return {
        data: data
      };
    }

    // handle money value provided as a string
    if (!isValidValue(value)) return result;
    var matchedMoney = _constants["default"].field.money.exec(value);
    if (!matchedMoney) {
      result.error = "Invalid money - Cannot parse money ".concat(value);
      return result;
    }
    result.data = {
      currencyCode: matchedMoney[1].toUpperCase(),
      centAmount: parseInt(matchedMoney[2], 10)
    };
    return result;
  }
  function mapSet(value, elementType) {
    var _this2 = this;
    var result = {
      error: [],
      data: []
    };
    var values = value.split(',');
    var _result = values.map(function (item) {
      var _item = item.trim();
      switch (elementType.name) {
        case 'Number':
          {
            return _this2.mapNumber(_item);
          }
        case 'Boolean':
          {
            return _this2.mapBoolean(_item);
          }
        case 'Money':
          {
            return _this2.mapMoney(_item);
          }
        case 'String':
        case 'Enum':
        case 'LocalizedEnum':
        case 'Date':
        case 'Time':
        case 'DateTime':
        case 'Reference':
          {
            return {
              data: _item
            };
          }
        default:
          {
            var unsupportedMsg = "'".concat(elementType.name, "' type is not supported! Kindly raise an issue for this");
            return {
              error: unsupportedMsg
            };
          }
      }
    });
    return _result.reduce(function (prev, curr) {
      if (curr.error) prev.error.push(curr.error);
      if (!(curr.data === undefined)) prev.data.push(curr.data);
      return prev;
    }, result);
  }
  function mapNumber(rawNo) {
    var _ref, _rawNo;
    var result = {};
    if (!isValidValue(rawNo)) return result;
    if (_ref = (_rawNo = rawNo, Number(_rawNo)), Number.isNaN(_ref)) {
      result.error = "The number ".concat(rawNo, " isn't valid");
      return result;
    }
    result.data = Number(rawNo);
    return result;
  }

  // Public methods
  return {
    parse: parse,
    mapBoolean: mapBoolean,
    mapMoney: mapMoney,
    mapSet: mapSet,
    mapNumber: mapNumber
  };
}();